/* Bucket behaviours */

//The sound made when a bucket is emptied/filled
const int SOUND_BUCKET_EMPTY = 40;

//Rewrote Bucket to offload specific target behaviours to separate functions. 
//New behaviour functions must still be called from this function, by putting the appropriate code in the SCRIPTED code block.

extern gotoCow 0xB18(var cow, var eventid);

extern useBucketOnNPC 0xB0B();
extern useBucketOnTrough 0xB0C();
extern useBucketOnDousable 0xB0D();
extern useBucketOnDough 0xB0E();
extern useBucketOnWell 0xB0F();
extern useBucketOnGround 0xB10();
Bucket 810 ()
{
	var bucket_frame;

	var target;
	var target_shape;
	var target_frame;
	var target_offsetx;
	var target_offsety;

	//User doubleclicked on bucket: march them to the bucket and pick it up, then call this function as event = SCRIPTED
	if (event == DOUBLECLICK)
	{
		//watering cans don't do anything
		if (UI_get_item_frame(item) == FRAME_WATERINGCAN) return;
		else gotoAndGet(item);
	}

	//This event level is triggered when the bucket is carried by the avatar and is now ready to be used on something (it shows the use cursor)
	if (event == SCRIPTED)
	{
		UI_close_gumps();

		bucket_frame = UI_get_item_frame(item);

		target = UI_click_on_item();
		target_shape = UI_get_item_shape(target);
		target_frame = UI_get_item_frame(target);

		//Player clicked on him/herself
		if (isAvatar(target))
		{
			if (bucket_frame == FRAME_BUCKET_BLOOD) avatarBark("\"No, thank thee.\"");
			else if (bucket_frame == FRAME_BUCKET_EMPTY) avatarBark("\"The bucket is empty.\"");
			else
			{
				avatarBark("\"Ahhh, how refreshing.\"");
				//Empty the bucket
				UI_set_item_frame(item, 0);
			}
			return;
		}

		//Player clicked on an NPC
		else if (UI_is_npc(target))
		{
			//Special behaviour for cows
			if (target_shape == SHAPE_COW)
			{
				if (bucket_frame == FRAME_BUCKET_EMPTY) gotoCow(target, MILK_WITH_BUCKET);
				else
				{
					avatarBark("\"The bucket is full.\"");
					return;
				}
			}
			else
			{
				if (bucket_frame == FRAME_BUCKET_EMPTY)
				{
					avatarBark("\"The bucket is empty.\"");
					return;
				}

				target_offsetx = [0, 2, 0, -2];
				target_offsety = [2, 0, -2, 0];

				//what this is doing is waiting 1 tick, for 50 loops. Why didn't they just use wait 50?
				UI_execute_usecode_array(target, [0x27, 1, 0x0B, -2, 0x0032]);
				gotoObject(target, target_offsetx, target_offsety, 0, useBucketOnNPC, target, SCRIPTED);
			}
		}

		//Player clicked on a churn
		else if (target_shape == SHAPE_KITCHEN_ITEM && target_frame == FRAME_CHURN)
		{
			if		(bucket_frame == FRAME_BUCKET_MILK) gotoChurn(target, CHURN_WITH_BUCKET);
			else if (bucket_frame == FRAME_BUCKET_EMPTY)
			{
				avatarBark("\"The bucket is empty.\"");
				return;
			}
			else return;
		}

		//Player clicked on a trough
		else if (target_shape in [SHAPE_TROUGH_HORIZONTAL, SHAPE_TROUGH_VERTICAL])
		{
			if (!(bucket_frame in [FRAME_BUCKET_EMPTY, FRAME_BUCKET_WATER])) return;

			if (target_shape == SHAPE_TROUGH_HORIZONTAL)
			{
				target_offsetx = [-1, -2, -1, -2, 1, 1, -4, -4];
				target_offsety = [1, 1, -2, -2, 0, -1, 0, -1];
			}
			else
			{
				target_offsetx = [1, 1, -2, -2, 0, -1, 0, -1];
				target_offsety = [-1, -2, -1, -2, 1, 1, -4, -4];
			}
			gotoObject(target, target_offsetx, target_offsety, 0, useBucketOnTrough, item, SCRIPTED);
		}

		//player clicked on a firepit
		else if (target_shape == SHAPE_FIREPIT)
		{
			//Now this is odd...only golden firepits can be used this way. It must be a FV quest thing.
			if (UI_get_item_frame(target) < 4 || UI_get_item_frame(target) > 7) return;

			if (bucket_frame == FRAME_BUCKET_EMPTY)
			{
				avatarBark("\"The bucket is empty.\"");
				return;
			}
			else if (bucket_frame != FRAME_BUCKET_WATER) return;

			target_offsetx = [-1, -2, 1, 1, -1, -2, -4, -4];
			target_offsety = [1, 1, -1, -2, -4, -4, -1, -2];

			gotoObject(target, target_offsetx, target_offsety, 0, useBucketOnDousable, target, SCRIPTED);
		}

		//player clicked on dousable lightsource
		else if (target_shape in [SHAPE_LIGHTSOURCE_LIT, SHAPE_SCONCE_LIT, SHAPE_TORCH_LIT, SHAPE_CAMPFIRE])
		{
			if (bucket_frame == FRAME_BUCKET_EMPTY)
			{
				avatarBark("\"The bucket is empty.\"");
				return;
			}
			else if (bucket_frame != FRAME_BUCKET_WATER) return;

			target_offsetx = [2, 0, -2, 0];
			target_offsety = [0, 2, 0, -2];

			gotoObject(target, target_offsetx, target_offsety, -5, useBucketOnDousable, target, SCRIPTED);
		}

		//player clicked on some flour
		else if (target_shape == SHAPE_DOUGH)
		{
			if (bucket_frame == FRAME_BUCKET_EMPTY)
			{
				avatarBark("\"The bucket is empty.\"");
				return;
			}
			else if (bucket_frame != FRAME_BUCKET_WATER) return;

			target_offsetx = [2, 0, -2, 0];
			target_offsety = [0, 2, 0, -2];

			gotoObject(target, target_offsetx, target_offsety, -5, useBucketOnDough, target, SCRIPTED);
		}


		//player clicked on well/well base
		else if (target_shape in [SHAPE_WELL, SHAPE_WELLBASE])
		{
			if (bucket_frame != FRAME_BUCKET_EMPTY)
			{
				avatarBark("\"The bucket is full.\"");
				return;
			}

			//they clicked on the well assembly - find the nearest base instead
			if (target_shape == SHAPE_WELL)
			{
				target = UI_find_nearest(target, SHAPE_WELLBASE, 3);
				if (!target) return;
			}

			target_offsetx = [-5, -5];
			target_offsety = [-1, -1];
			gotoObject(target, target_offsetx, target_offsety, 0, useBucketOnWell, item, SCRIPTED);
		}
		
		//player clicked on a rock (this is for FV quest) or on the ground
		else if (target_shape == SHAPE_ROCK || target[1] == 0)
		{
			if (bucket_frame == FRAME_BUCKET_EMPTY)
			{
				avatarBark("\"The bucket is empty.\"");
				return;
			}

			//This strips the object reference from target, making it a regular X,Y,Z coordinate array
			target = removeFromArray(target[1], target);

			target[Y] = target[Y] + 1;
			UI_path_run_usecode(target, useBucketOnGround, item, SCRIPTED);
		}
	}
}


//Bucket was used on an NPC (this behaviour was moved from Bucket() to aid code organisation)
//Todo: use event to specify what kind of bucket was used - so we can avoid 'accidental' dousings in blood
//Updated 2005-03-18 to a) wake NPC up and b) make sure NPC can talk before making them bark
extern makeSpill 0x694();
useBucketOnNPC 0xB0B()
{
	//Note: item is the NPC clicked on

	var bucket;				//the first bucket we can find on the player
	var bucket_frame;		//the frame of that bucket
	var direction;			//the direction of the NPC, relative to the avatar
	var towards_avatar;	//the direction of the avatar, relative to the NPC
	var bark;				//what the NPC says when you try it on

	//get the first bucket we can find that the avatar is carrying
	bucket = UI_get_cont_items(UI_get_avatar_ref(), SHAPE_BUCKET, QUALITY_ANY, FRAME_ANY);
	bucket_frame = UI_get_item_frame(bucket);

	direction = directionFromAvatar(item);
	towards_avatar = invertDirection(direction);

	//wake the person up
	//Todo: this should happen after the dousing, rather than before
	//Note: most of the time it's impossible to actually get to an NPC who's sleeping in a bed
	UI_clear_item_flag(item, ASLEEP);

	//make sure NPC can talk
	if (canTalk(item))
	{
		if (bucket_frame == FRAME_BUCKET_BLOOD) bark = "\"Foul miscreant!\"";
		else bark = "\"Hey, stop that!\"";
	}

	//Plays the avatar tossing water onto the NPC
	script AVATAR
	{
		face direction;
		actor frame SWING_3;
		actor frame USE;
		actor frame STAND;
	}

	//target getting peevish
	script item
	{
		face towards_avatar;
		wait 2;
		say bark;
		wait 5;
	}

	//empty the bucket
	//Added: water is now spilled!
	script bucket	{ wait 2; call makeSpill; }
}

//Bucket was used on a trough (this behaviour was moved from Bucket() to aid code organisation)
useBucketOnTrough 0xB0C()
{
	var trough;				//the nearest trough
	var trough_frame;		//the trough's frame when it was clicked on
	var trough_new_frame;	//the trough's frame after the bucket has been used on it
	var bucket_frame;		//the bucket's frame when it was used
	var bucket_new_frame;	//the bucket's frame after it was used
	var direction;			//direction to the trough, relative to avatar (used for animation)

	trough = false;
	//try to find the nearest horizontal trough
	trough = UI_find_nearest(UI_get_avatar_ref(), SHAPE_TROUGH_HORIZONTAL, 5);
	//if we couldn't find one, try the nearest vertical trough
	if (!trough) trough = UI_find_nearest(UI_get_avatar_ref(), SHAPE_TROUGH_VERTICAL, 5);
	//no available trough, alas!
	if (!trough) return;

	bucket_frame = UI_get_item_frame(item);
	trough_frame = UI_get_item_frame(trough);

	//bucket is full of water, fill the trough from the bucket
	if (bucket_frame == FRAME_BUCKET_WATER)
	{
		//trough is already full
		if (trough_frame == 3 || trough_frame == 7)
		{
			avatarBark("\"The trough is full.\"");
			return;
		}

		//increment the trough's fullness
		trough_new_frame = trough_frame + 1;
		bucket_new_frame = FRAME_BUCKET_EMPTY;
	}

	//bucket is empty, fill the bucket from the trough
	//(I can't believe someone got paid to implement this level of interactivity.)
	else if (bucket_frame == FRAME_BUCKET_EMPTY)
	{
		//no more water for you!
		if (trough_frame == 0 || trough_frame == 4)
		{
			avatarBark("\"The trough is empty.\"");
			return;
		}
		
		//decrement the trough's fullness
		trough_new_frame = trough_frame - 1;
		bucket_new_frame = FRAME_BUCKET_WATER;
	}
	//bucket was full of something other than water, bail out
	else return;

	direction = directionFromAvatar(trough);

	//lean down in the direction of the trough
	script AVATAR
	{
		face direction;
		actor frame LEAN;
		wait 2;
		actor frame STAND;
	}

	//change the trough's frame
	script trough
	{
		wait 2;
		frame trough_new_frame;
		continue;
		sfx SOUND_BUCKET_EMPTY;
	}

	//change the bucket's frame
	script item	{ wait 2; frame bucket_new_frame; }
}

//Bucket was used on a dousable item
useBucketOnDousable 0xB0D()
{
	//Note: item is the object clicked on

	var bucket;
	var bucket_frame;
	var target_shape;
	var target_frame;
	var target_direction;

	var new_shape;
	var new_lightsource;
	var lightsource_quality;

	var target_pos;
	var sprite_pos;
	var sprite_offset;


	//get the first bucket of water we can find in the player's inventory
	bucket = UI_get_cont_items(UI_get_avatar_ref(), SHAPE_BUCKET, QUALITY_ANY, FRAME_BUCKET_WATER);
	bucket_frame = UI_get_item_frame(bucket);

	target_shape = UI_get_item_shape(item);
	target_frame = UI_get_item_frame(item);

	target_direction = directionFromAvatar(item);
	target_pos = UI_get_object_position(item);

	if (target_shape == SHAPE_FIREPIT)
	{
		if (target_frame == 4)
		{
			avatarBark("\"There are only coals.\"");
			return;
		}
		else if (target_frame == 7) UI_execute_usecode_array(item, [0x55, 0x0693, 0x01, 0x50, 0x50, 0x50]);
		else if (target_frame == 6) UI_execute_usecode_array(item, [0x55, 0x0693, 0x01, 0x50, 0x50]);
		else if (target_frame == 5) UI_execute_usecode_array(item, [0x55, 0x0693, 0x01, 0x50]);

		UI_execute_usecode_array(UI_get_avatar_ref(), [0x59, target_direction, 0x6C, 0x27, 2, 0x61]);
	}

	else if (target_shape == SHAPE_LIGHTSOURCE_LIT || target_shape == SHAPE_SCONCE_LIT || target_shape == SHAPE_TORCH_LIT)
	{

		if (target_frame == 16)	//the last style of light source has some special behaviour (FV item?)
		{
			sprite_offset = 2;
			UI_execute_usecode_array(UI_get_avatar_ref(), [0x59, target_direction, 0x67, 0x64, 0x61, 0x27, 1, 0x52, "\"I can't douse it.\""]);
		}
		else
		{
			sprite_offset = 3;
			lightsource_quality = UI_get_item_quality(item);

			//determine the correct doused shape
			if		(target_shape == SHAPE_LIGHTSOURCE_LIT)	new_shape = SHAPE_LIGHTSOURCE;
			else if	(target_shape == SHAPE_SCONCE_LIT)		new_shape = SHAPE_SCONCE;
			else if	(target_shape == SHAPE_TORCH_LIT)		new_shape = SHAPE_TORCH;

			//remove the old lightsource and put a new one in its place
			UI_remove_item(item);

			new_lightsource = UI_create_new_object(new_shape);
			UI_set_item_quality(new_lightsource, lightsource_quality);
			UI_set_item_frame(new_lightsource, target_frame);
			UI_update_last_created(target_pos);

			//play lean-down animation?
			UI_execute_usecode_array(UI_get_avatar_ref(), [0x59, target_direction, 0x67, 0x64, 0x61]);
		}

		//place the extinguishing effect relative to the elevation of the light source
		//Todo: separate this as a generic function, since I can see this being used often for sprite effects

		sprite_offset = sprite_offset + (target_pos[Z] / 2);
		sprite_pos[X] = (target_pos[X] - sprite_offset);
		sprite_pos[Y] = (target_pos[Y] - sprite_offset);

		//display the puff of smoke
		UI_sprite_effect(9, sprite_pos[X], sprite_pos[Y], 0, 0, 0, -1);

		//tfssss
		UI_play_sound_effect(0x002E);
	}

	else if (target_shape == SHAPE_CAMPFIRE)
	{
		if (target_frame == 0)
		{
			avatarBark("\"There are only coals.\"");
			return;
		}

		//snuff out the fire
		UI_execute_usecode_array(item, [0x46, 0]);

		//bend down?
		UI_execute_usecode_array(UI_get_avatar_ref(), [0x59, target_direction, 0x6C, 0x27, 2, 0x61]);

		UI_sprite_effect(9, target_pos[X], target_pos[Y], 0, 0, 0, -1);
		UI_play_sound_effect(0x002E);
	}
}

//Bucket of water was used on dough flour
//Note: this doesn't actually decrement the bucket's quality or anything, which means you can use the bucket as many times as you like
useBucketOnDough 0xB0E()
{
	var direction;

	//The dough has already been watered
	if (UI_get_item_frame(item) != 0) return;

	direction = directionFromAvatar(item);
	//lean down in direction of dough?
	UI_execute_usecode_array(UI_get_avatar_ref(), [0x59, direction, 0x6C, 0x27, 2, 0x61]);
	UI_set_item_frame(item, FRAME_DOUGH_BALL);
}

//Bucket of water was used on well
useBucketOnWell 0xB0F()
{
	var well;
	var well_frame;

	//find the nearest well
	well = UI_find_nearest(UI_get_avatar_ref(), SHAPE_WELL, 10);
	well_frame = UI_get_item_frame(well);

	//well is of the regular style
	if (well_frame >= 0 && well_frame <= 11) well_frame = 1;

	//well is a golden FV jobbie
	else if (well_frame >= 12 || well_frame <= 23) well_frame = 13;

	//animate the well
	UI_execute_usecode_array(well, [0x27, 1, 0x46, well_frame, 0x27, 2, 0x4E, 0x27, 1, 0x4E, 0x27, 1, 0x4E, 0x27, 1, 0x4E, 0x27, 1, 0x4E]);

	//animate the avatar
	UI_execute_usecode_array(UI_get_avatar_ref(), [0x59, 2, 0x67, 0x27, 2, 0x59, 4, 0x27, 1, 0x65, 0x27, 1, 0x66, 0x27, 1, 0x64, 0x27, 1, 0x67, 0x27, 4]);

	//place the full bucket nearby
	UI_execute_usecode_array(item, [0x27, 0x0011, 0x55, 0x0695]);
}

//Bucket was used on a rock (FV quest) or on the ground
useBucketOnGround 0xB10()
{
	UI_execute_usecode_array(UI_get_avatar_ref(), [0x59, 0, 0x6C, 0x27, 3, 0x61]);
	UI_execute_usecode_array(item, [0x27, 3, 0x55, 0x0694]);
}


//reimplementation of internal function - creates blood/winestains
//Added in special stain for milk
makeSpill 0x694()
{
	var bucket_frame;
	var min_frame;
	var max_frame;
	var avatar_pos;
	var target_pos;
	var spill;
	var bucket_quality;

	bucket_frame = UI_get_item_frame(item);
	min_frame = 0;
	max_frame = 0;

	if (bucket_frame == FRAME_BUCKET_WATER)
	{
		min_frame = 16;
		max_frame = 19;
	}
	else if (bucket_frame == FRAME_BUCKET_BLOOD)
	{
		min_frame = 0;
		max_frame = 3;
	}
	else if (bucket_frame == FRAME_BUCKET_WINE)
	{
		min_frame = 20;
		max_frame = 23;
	}
	else if (bucket_frame in [FRAME_BUCKET_BEER, FRAME_BUCKET_BEER2, FRAME_BUCKET_MILK])
	{
		min_frame = 12;
		max_frame = 15;
	}
	else if (bucket_frame == FRAME_WATERINGCAN) return;

	avatar_pos = UI_get_object_position(UI_get_avatar_ref());

	target_pos[X] = avatar_pos[X];
	target_pos[Y] = avatar_pos[Y] - 1;
	target_pos[Z] = 0;

	spill = UI_create_new_object(SHAPE_SPILL);
	UI_set_item_frame(spill, UI_die_roll(min_frame, max_frame));	//choose a random frame
	UI_set_item_flag(spill, TEMPORARY);	//added to the original, prevents them sticking around FOREVER AND EVER

	UI_update_last_created(target_pos);

	//for FV quest: decrement the quality (ie, remaining spills) of the bucket
	bucket_quality = UI_get_item_quality(item);
	if(bucket_frame == FRAME_BUCKET_BLOOD && bucket_quality)
	{
		UI_set_item_quality(item, bucket_quality - 1);
	}
	//switch the bucket to the empty frame
	else UI_set_item_frame(item, FRAME_BUCKET_EMPTY);

	//ffssshhhh
	UI_play_sound_effect(SOUND_BUCKET_EMPTY);
}