const int QUALITY_FRESHFISH	= 1;	//the quality of freshly-caught fish (this will expire to 0 at EXPIRY_RATE)
const int FISH_EXPIRY_RATE	= 50;	//the quality will be decremented by one every EXPIRY_RATE ticks
//These factors are not currently used: sleeping will not decrement quality properly.

const int BASE_CATCH_CHANCE = 30;	//% base chance of catching a fish, if any are about
const int CHANCE_PER_FISH	= 10;	//% extra catch chance per nearby fish

const int SOUND_LANDEDFISH	= 40;	//played when you catch a fish
const int SOUND_FISHING		= 103;	//the plop of bait entering the water

//Reconstituted from original function, with some extra bits added (more advice to the player, tweaked catch chance)
//Also extended animation considerably, and increased catch chance to compensate for the delay
FishingRod 662 ()
{
	var target;
	var avatar_pos;
	var fish_caught;
	var num_fish_nearby;
	var nearby_fish;
	var fish;
	var caught_fish;
	var rand;

	//Rod was doubleclicked on: start fishing
	if (event == DOUBLECLICK)
	{
		//make sure the fishing rod is readied first
		if (!UI_is_readied(AVATAR, WEAPON_HAND, SHAPE_FISHING_ROD, FRAME_ANY))
		{
			randomPartySay("\"Thou must have the rod in thy hand.\"");
			return;
		}

		UI_close_gumps();
		target = UI_click_on_item();

		if (!(UI_is_water([target[2], target[3], target[4]]) || UI_get_item_shape(target) == SHAPE_FISH))
		{
			UI_flash_mouse(0);

			//only make this 1 in 4, to stop irritating the player
			if (UI_get_random(4) == 1)
			{
				//Spark is the resident angling expert
				if (inParty(SPARK)) SPARK.say("\"Thou shouldst cast thy line into the water, Avatar!\" He grins cheekily.");
				else randomPartySay("\"Last I heard, fish doth not live upon dry land. Thou shouldst try thy rod in the water!\"");
			}
			return;
		}
		else
		{
			//determine which way the avatar should be facing
			var direction = directionFromAvatar(target);
			//fishing animation
			script AVATAR
			{
				nohalt;
				call freeze;
				face direction;
				actor frame SWING_2H_2;
				actor frame SWING_2H_3;
				sfx SOUND_FISHING;

				//prevents the animation 'timing out' and reverting to standing
				repeat 2	{ actor frame SWING_2H_3; wait 5; };

				call FishingRod;	//call this function again, as event SCRIPTED, to determine if fish were caught
				actor frame USE;
				actor frame STAND;
				call unfreeze;
			}
		}
	}

	else if (event == SCRIPTED)
	{
		nearby_fish		= UI_find_nearby(AVATAR, SHAPE_FISH, 15, 0);
		num_fish_nearby	= UI_get_array_size(nearby_fish);

		//Adjusted fish-catching chance: now it is a base chance + an adjustable chance per fish
		if (num_fish_nearby > 0)
		{
			var catch_chance = BASE_CATCH_CHANCE + (num_fish_nearby * CHANCE_PER_FISH);
			if (UI_get_random(100) <= catch_chance) fish_caught = true;
		}

		//Hooray, we got one!
		if (fish_caught)
		{
			fish = UI_create_new_object(SHAPE_FOOD);
			if (!fish) return;

			UI_set_item_frame(fish, FRAME_TROUT);
			UI_set_item_flag(fish, TEMPORARY);		//Added to stop fish sticking around indefinitely

			//put it near the avatar's feet
			avatar_pos = UI_get_object_position(AVATAR);
			avatar_pos[X] = avatar_pos[X] + 1;
			UI_update_last_created(avatar_pos);

			UI_play_sound_effect(SOUND_LANDEDFISH);

			//Added: indicates that the fish is fresh (for sale to Gordon)
			//Note: this required a new tfa.dat, to change the class of the Food shape from Quality Flags to Quality.
			UI_set_item_quality(fish, QUALITY_FRESHFISH);

			//Added: start the fish quality decrementing
			//Removed: disabled for now, as it doesn't count sleep at all
			//script fish after FISH_EXPIRY_RATE ticks { call 0xB40; }


			//Added: remove a fish from the water
			if (UI_get_item_shape(target) == SHAPE_FISH) caught_fish = target;	//Player clicked on a particular fish - grab that one
			else caught_fish = nearby_fish[UI_get_random(num_fish_nearby)];		//otherwise, grab one of the nearby fishies at random
			UI_remove_item(caught_fish);

			//Make one of the party members comment on the catch
			rand = UI_get_random(3);
			if (rand == 1)
			{
				randomPartyBark("\"Indeed, a whopper!\"");
				if (UI_npc_nearby(SPARK)) delayedBark(SPARK, "\"I have seen bigger.\"", 16);
			}
			else if (rand == 2) randomPartyBark("\"What a meal!\"");
			else if (rand == 3)	randomPartyBark(["\"That fish does not", "look right.\""]);
		}

		//changed barks to reflect that there's no fish (better player guidance)
		else if (num_fish_nearby == 0)
		{
			rand = UI_get_random(3);
			if (rand == 1)
			{
				//Spark, butting his head in as usual
				if (inParty(SPARK) && canTalk(SPARK))
				{
					SPARK.say("\"The fish doth not seem to be biting, thou shouldst try somewhere else!~\"My father took me fishing sometimes -- thou canst find good spots near bridges, where thou canst see the fishes beneath the water.\"");
				}
				else randomPartySay("\"Methinks there are no fish in these waters. Let us try in a better spot!\"");
			}
			else if (rand == 2) delayedBark(AVATAR, "\"Not even a bite!\"", 0);
			else if (rand == 3) delayedBark(AVATAR, "\"I've lost my bait.\"", 0);
		}

		else
		{
			rand = UI_get_random(4);

			if (rand == 1) delayedBark(AVATAR, "\"Not even a bite!\"", 0);
			else if (rand == 2)
			{
				delayedBark(AVATAR, "\"It got away!\"", 1);
				if (UI_npc_nearby(IOLO)) delayedBark(IOLO, "\"It was the Big One!\"", 16);
			}
			else if (rand == 3) delayedBark(AVATAR, "\"I've lost my bait.\"", 0);
			else if (rand == 4) delayedBark(AVATAR, "\"I felt a nibble.\"", 0);
		}
	}
}

//This function is responsible for making fish go bad after a certain length of time
//Not used: sleeping will not advance the script queue, and the intrinsics for torches that work around this are not yet implemented in Exult.
expireFish 0xB40()
{
	var reduced_quality = UI_get_item_quality(item) - 1;
	UI_set_item_quality(item, reduced_quality);

	if (reduced_quality == 0)
	{
		UI_halt_scheduled(item);	//stop the countdown
		avatarBark("Fish now rotten!");
	}
	else
	{
		avatarBark("Fish now at " + reduced_quality + " quality");
		UI_delayed_execute_usecode_array(item, [0x55, 0x0600], FISH_EXPIRY_RATE);
	}
}