/* Added new behaviour to keys: Specifically, if you use them on the Avatar you will get a description of the key */

const int SOUND_KEY = 27;	//Snick

const int KEY_USED = 28;	//Flag used for marking a key as having been used (same as KEY_USED flag actually)

//Known key IDs
enum key_ids
{
	KEY_PICKABLE			= 0,	//indicates the chest/door is pickable using lockpicks
									//(there should be a Skeleton Key with this ID floating around somewhere too)
	KEY_PICKABLE_TRAPPED	= 255,	//only found on chests - doors cannot be trapped.
									//this means that inn keys can unlock trapped chests!
	KEY_INN					= 255,	//keys for inn doors. These were meant to be auto-removed after use or after leaving the inn,
									//but apparently they never implemented this.

	KEY_PIRATES_CHEST			= 99,	//Chest in the secret pirate's hideout east of Jhelom (found nearby)
	KEY_PIRATES_CHEST_2			= 100,	//Chests in pirate cave west of Jhelom (found nearby)
	KEY_GARRITTS_CHEST			= 224,	//Garritt's chest in the Fellowship Shelter in Paws (given by Morfin)
	KEY_CASTLEBRITISH_STOREROOM	= 236,	//Storeroom in Castle British (found in a nightstand downstairs)
	KEY_MORFINS_CHEST			= 242,	//Morfin's chest in his strong-room in paws (found under bucket)
	KEY_MORFINS_STRONGROOM		= 245,	//Morfin's strong-room in paws (found hidden in his house)
	KEY_CHRISTOPHERS_CHEST		= 253	//Christopher's chest in Trinsic (found beside his murdered body)
};

extern useKeyOnDoor 0x815(var door);
extern useKeyOnChest 0xB14(var chest);
Key 641 ()
{
	var target;			//item key was used on (e.g. door/chest)
	var target_shape;	//shape of the item key was used on
	var key_id;			//key id of key
	var target_id;		//key id of door
	var msg;			//key description message
	var key_used;		//whether or not the key has been used successfully to open a door (changes the description messages)

	if (event != DOUBLECLICK) return;

	target = UI_click_on_item();
	UI_play_sound_effect2(SOUND_KEY, item);

	target_shape	= UI_get_item_shape(target);
	key_id			= UI_get_item_quality(item);
	target_id		= UI_get_item_quality(target);

	//Key used on a door (Why target[1]?? Shouldn't just target work?)
	if (target_shape in [SHAPE_DOOR_HORIZONTAL, SHAPE_DOOR_VERTICAL, SHAPE_DOOR2_HORIZONTAL, SHAPE_DOOR2_VERTICAL])
	{
		if (key_id == target_id)
		{
			useKeyOnDoor(target);
			UI_set_item_flag(item, KEY_USED);
		}
		else UI_item_say(target, "Doesn't fit");
	}

	//Key used on a chest (this used to be built into Key(), but has been separated off to useKeyOnChest() instead
	else if (target_shape in [SHAPE_CHEST, SHAPE_LOCKED_CHEST])
	{
		if (key_id == target_id)
		{
			useKeyOnChest(target);
			UI_set_item_flag(item, KEY_USED);
		}
		else UI_item_say(target, "Doesn't fit");
	}

	//Key was used on the avatar: provide a key description
	//Todo: write some quick description-generation functions to make this less tedious?
	else if (target == UI_get_avatar_ref())
	{
		key_used = UI_get_item_flag(item, KEY_USED);

		//Skeleton key
		if (key_id == KEY_PICKABLE)
		{
			if (key_used) msg = "This is a skeleton key: it can open any lock that can be picked.";
			else msg = "This key is oddly cut, with many more teeth than usual. You suspect that it is a skeleton key, which could open any pickable lock.";
		}

		//General inn key
		else if (key_id == KEY_INN) msg = "You were given this key by the innkeeper. No doubt the establishment will want you to leave it when you go.";

		else if (key_id == KEY_CHRISTOPHERS_CHEST)
		{
			//Let Spark describe the key instead
			if (isNearby(SPARK))
			{
				SPARK.say("\"I recognise that key! 'Tis my father's, and opens his chest in our house.\"");
				return;
			}
			else
			{
				//Player has already opened the chest (go go quest flags)
				if (key_used || gflags[UNLOCKED_CHRISTOPHERS_CHEST]) msg = "This key fitted the chest in the home of Christopher the blacksmith, in Trinsic.";
				else msg = "You found this key beside a murder victim in Trinsic. It looks small enough to fit a chest.";
			}
		}

		else if (key_id == KEY_GARRITTS_CHEST)
		{
			if (key_used) msg = "This key opened Garritt's chest in the Fellowhsip Shelter in Paws.";
			else msg = "Morfin gave you this key, saying that the boy Garritt had dropped it while 'playing' near his slaughterhouse in Paws. It looks too small to fit a door.";
		}

		else if (key_id == KEY_CASTLEBRITISH_STOREROOM)
		{
			if (key_used) msg = "This key fits the storeroom door on the second floor of Castle British.";
			else if (hasMet(LORD_BRITISH)) msg = "You found this key in Castle British. It must be the one Lord British mentioned, that opens the castle storeroom.";
			else msg = "You found this key in Castle British. It looks like it would open a sturdy KEY_USEDal door.";
		}

		else if (key_id == KEY_MORFINS_CHEST)
		{
			if (key_used) msg = "This key fits a chest in Morfin's slaughterhouse in Paws.";
			else msg = "This key was hidden in Morfin's slaughterhouse in Paws. It looks like it would fit a chest.";
		}

		else if (key_id == KEY_MORFINS_STRONGROOM)
		{
			if (key_used) msg = "This key fits the door to the strongroom in the Paws slaughterhouse.";
			else msg = "You found this key in the home of Morfin the slaughterhouse owner, in Paws.";
		}

		else if (key_id == KEY_PIRATES_CHEST)
		{
			if (key_used) msg = "This key opens a chest in a pirate's hideout east of Jhelom.";
			else msg = "You found this key stashed in a pirate's hideout east of Jhelom. It is shaped to fit a chest.";
		}

		else if (key_id == KEY_PIRATES_CHEST_2) msg = "You found this key stashed in a pirate hideout west of Jhelom. It is shaped to fit a chest.";

		if (msg) avatarSay(msg);
	}

	//debug behaviour: display the key id so we can make an appropriate description for it
	//else avatarBark("Key id: " + key_id);
}

//Shifted this behaviour from Key() to an external function for cleaner code organisation
useKeyOnChest 0xB14(var chest)
{
	var chest_locked;
	var key_id;
	var target_id;
	var key_inside_chest;

	chest_locked = (UI_get_item_shape(chest) == SHAPE_LOCKED_CHEST);
	key_id = UI_get_item_quality(item);
	target_id = UI_get_item_quality(chest);

	//Unlock chest
	if (chest_locked)
	{
		UI_item_say(chest, "Unlocked");
		UI_set_item_shape(chest, SHAPE_CHEST);

		if (target_id == KEY_CHRISTOPHERS_CHEST) gflags[UNLOCKED_CHRISTOPHERS_CHEST] = true;
	}

	//Lock the chest
	else
	{
		//Make sure the key isn't actually *in* the chest
		if (containedBy(item, chest))
		{
			//Dunno what item -361 corresponds to yet (Current object? But why?)
			//UI_item_say(-361, "Key inside");
			//Going with item for now
			UI_item_say(item, "Key inside");
			return;
		}
		else
		{
			//UI_UNKNOWN_80(chest);	//This closes the gump corresponding to the chest. Argh, how can we call this if it isn't defined??
			UI_close_gumps();		//We do it this way for now. Find out how to define missing intrinsics!
			UI_set_item_shape(chest, SHAPE_LOCKED_CHEST);
			UI_item_say(chest, "Locked");
		}
	}
}

//The different states a door can be in
enum door_states
{
	DOOR_UNLOCKED		= 0,	//Unlocked but closed (vertical latch)
	DOOR_OPEN			= 1,	//Open (horizontal latch)
	DOOR_LOCKED			= 2,	//Locked (vertical latch, circle below)
	DOOR_LOCKED_MAGIC	= 3		//Locked with magic spell (glowing latch)
};

extern var getDoorState 0x81B(var door);			//Checks the door's current lock state (compare with the constants above)
extern setDoorState 0x81C(var door, var new_state);	//Sets the door to the correct state

useKeyOnDoor 0x815 (var door)
{
	var door_state;
	var key_id;
	var target_id;

	door_state	= getDoorState(door);
	key_id		= UI_get_item_quality(item);
	target_id	= UI_get_item_quality(item);

	if (door_state == DOOR_UNLOCKED)
	{
		//Todo: find out what these flags correspond to
		if		(key_id == 228) gflags[737] = false;
		else if (key_id == 247) gflags[738] = false;

		//Inn keys should be deleted after they have unlocked a door
		//Disabled, since this will leave inn doors permanently unlocked. Instead, change innkeeper conversations so that they ask for the key back when the player tries to leave.
		//else if (key_id == 255) UI_remove_item(item);

		setDoorState(door, DOOR_LOCKED);
	}

	else if (door_state == DOOR_OPEN)
	{
		randomPartySay("\"Excuse me, the door is already open. " + "Is it not rather futile to lock it now?\"");
	}

	else if (door_state == DOOR_LOCKED)
	{
		//Todo: find out what these flags correspond to
		if		(key_id == 228) gflags[737] = true;
		else if	(key_id == 247) gflags[738] = true;

		setDoorState(door, DOOR_UNLOCKED);
	}

	else if (door_state == DOOR_LOCKED_MAGIC)
	{
		if (UI_get_random(10) == 1)	//No idea why
		{
			randomPartySay("\"Excuse me, the door appears magically locked. " + "Is it not rather difficult to unlock it with a key?\"");
		}
	}
}

//reimplemented this original function, because it was easier than flipping arguments
setDoorState 0x81C (var door, var new_state)
{
	var current_frame;
	var current_state;

	current_frame = UI_get_item_frame(door);
	current_state = (current_frame % 4);
	UI_set_item_frame(door, current_frame - current_state + new_state);
}


/* Reimplemented for kicks */
Lockpick 0x273 ()
{
	var target;
	var target_shape;
	var target_id;
	var picked;
	var trap;

	if (event != DOUBLECLICK) return;

	target = UI_click_on_item();
	UI_play_sound_effect2(SOUND_KEY, item);

	target_shape	= UI_get_item_shape(target);
	target_id		= UI_get_item_quality(target);

	picked = (UI_get_random(30) < UI_get_npc_prop(UI_get_avatar_ref(), DEXTERITY));

	//picking locked chest
	if (target_shape == SHAPE_LOCKED_CHEST)
	{
		if (target_id == KEY_PICKABLE || target_id == KEY_PICKABLE_TRAPPED)	//regular locked chest
		{
			if (picked)
			{
				//seems trapped chests explode when you're successful, rather than when you're not
				if (target_id == KEY_PICKABLE_TRAPPED)
				{
					trap = UI_create_new_object(SHAPE_POWDER_KEG);
					if (!trap) return;
					UI_update_last_created(UI_get_object_position(target));
					UI_close_gumps();
					UI_explode(trap, trap, SHAPE_POWDER_KEG);
				}

				UI_set_item_shape(target, SHAPE_CHEST);
				UI_item_say(target, "Unlocked");
			}
			else
			{
				UI_item_say(target, "Pick broke");
				subtractQuantity(item);
			}
		}
		//Added to original behaviour
		else randomPartyBark("\"Strange that did not work.\"");
	}

	//Locked door
	else if (target_shape in [SHAPE_DOOR_HORIZONTAL, SHAPE_DOOR_VERTICAL, SHAPE_DOOR2_HORIZONTAL, SHAPE_DOOR2_VERTICAL])
	{
		if (getDoorState(target) != DOOR_LOCKED) return;

		else if (target_id != KEY_PICKABLE)
		{
			randomPartyBark("\"Strange that did not work.\"");
		}
		else if (picked)
		{
			setDoorState(target, DOOR_UNLOCKED);
			UI_item_say(target, "Unlocked");
		}
		else
		{
			UI_item_say(target, "Pick broke");
			subtractQuantity(item);
		}
	}
	//Pick used on soKEY_USEDhing else
	else
	{
		randomPartySay("\"Try those on a locked chest or door.\"");
	}
}