/* This script contains behaviors for kitchen items and buttern churning. Since bucket and milk-bottle behaviours use constants defined in this script, it must be included before bucket.uc and food.uc. */

//flags (event ids, actually) to specify what kind of milk container is being churned/milked with
enum milk_types
{
	CHURN_WITH_BOTTLE	= 1,
	CHURN_WITH_BUCKET	= 2,
	CHURN_WITH_PITCHER	= 3,

	MILK_WITH_BUCKET	= 2,
	MILK_WITH_PITCHER	= 3
};

extern gotoChurn 0xB13(var churn, var milktype);
extern gotoCow 0xB18(var cow, var milktype);

KitchenItem 863()
{
	var framenum;
	var target;
	var target_pos;
	var target_shape;
	var flour;

	framenum = UI_get_item_frame(item);

	if (event == DOUBLECLICK)
	{
		//flour sack behaviour
		if (framenum == FRAME_FLOURSACK_OPEN)
		{
			target			= UI_click_on_item();
			target_pos		= UI_get_object_position(target);
			target_shape	= UI_get_item_shape(target);

			if (target_shape == SHAPE_WORKTABLE_HORIZONTAL)
			{
				target_pos[X] = target_pos[X] - UI_die_roll(0, 3);
				//vertical offset ensures the flour ends up on top of the table
				target_pos[Z] = target_pos[Z] + 2;
			}
			else if (target_shape == SHAPE_WORKTABLE_VERTICAL)
			{
				target_pos[Y] = target_pos[Y] - UI_die_roll(0, 2);
				//vertical offset ensures the flour ends up on top of the table
				target_pos[Z] = target_pos[Z] + 2;
			}
			//only those two kinds of tables are supported, oddly
			else
			{
				randomPartySay("\"Why not put the flour on the table first?\"");
				return;
			}

			//create and position the new flour object
			flour = UI_create_new_object(SHAPE_DOUGH);
			if (!flour) return;
			UI_set_item_frame(flour, FRAME_FLOUR);
			UI_set_item_flag(flour, TEMPORARY);	//now what the hell is this flag?
			UI_update_last_created(target_pos);
		}

		//empty pitcher behaviour - go and pick up the pitcher
		//then rerun the function with event = SCRIPTED
		else if (framenum == FRAME_PITCHER) gotoAndGet(item);

		//rolling pin behaviour
		else if (framenum in [FRAME_ROLLINGPIN, FRAME_ROLLINGPIN_2])
		{
			target = UI_click_on_item();

			//rolling pin was used on a ball of dough
			if (UI_get_item_shape(target) == SHAPE_DOUGH && UI_get_item_frame(target) == FRAME_DOUGH_BALL)
			{
				UI_set_item_frame(target, FRAME_DOUGH_FLAT);	//roll it out into flat dough
			}

			//rolling pin was used on a person (donk!)
			else if (canTalk(target))
			{
				UI_item_say(target, "\"Hey! That really hurt!\"");
			}
		}

		//closed floursack behaviour (no idea why there are two identical frames for this...)
		else if (framenum in [FRAME_FLOURSACK, FRAME_FLOURSACK_2])
		{
			UI_set_item_frame(item, FRAME_FLOURSACK_OPEN);
		}

		//churn behaviour (simple prompt for milk)
		else if (framenum == FRAME_CHURN)
		{
			randomPartySay("\"Thou shan't get far without some milk to churn!\"");
		}

		//full pitcher behaviour
		else if (framenum == FRAME_PITCHER_MILK)
		{
			target = UI_click_on_item();
			if (UI_get_item_shape(target) == SHAPE_KITCHEN_ITEM && UI_get_item_frame(target) == FRAME_CHURN)
			{
				gotoChurn(target, CHURN_WITH_PITCHER);
			}
			else
			{
				randomPartySay("\"Why not churn that milk into butter?\"");
			}
		}
	}

	//currently only pitchers use this, since they need to be manually picked up
	//Note that this calls milkCow(), which lives in cow.uc
	else if (event == SCRIPTED)
	{
		if (framenum == FRAME_PITCHER)
		{
			target = UI_click_on_item();
			if (UI_get_item_shape(target) == SHAPE_COW) gotoCow(target, MILK_WITH_PITCHER);
			else
			{
				randomPartySay("\"Thou couldst always milk a cow to fill yon pitcher... if thou don't mind getting thy hands dirty, that is!\"");
			}
		}
	}
}


/* Butter-churning behaviours */

//Churn butter from milk
//----------------------

//Churns the butter
//This uses event as a flag to determine the source of the milk: 1 = milk bottle, 2 = bucket of milk, 3 = milk pitcher
const int CHURN_SOUND = 4;
extern createButter 0xB03();
churnButter 0xB02()
{
	var source;			//the source of the milk (either milk bottle or bucket of milk)

	//churned from bottle of milk
	if (event == CHURN_WITH_BOTTLE)
	{
		//Remove milk from the party's stash
		source = UI_get_cont_items(AVATAR, SHAPE_BOTTLE, QUALITY_ANY, FRAME_MILK);
		UI_remove_item(source);
	}
	//churned from bucket of milk
	else if (event == CHURN_WITH_BUCKET)
	{
		source = UI_get_cont_items(AVATAR, SHAPE_BUCKET, QUALITY_ANY, FRAME_BUCKET_MILK);
		//empty the bucket
		UI_set_item_frame(source, FRAME_BUCKET_EMPTY);
	}
	//churned from pitcher of milk
	else if (event == CHURN_WITH_PITCHER)
	{
		source = UI_get_cont_items(AVATAR, SHAPE_KITCHEN_ITEM, QUALITY_ANY, FRAME_PITCHER_MILK);
		//empty the pitcher
		UI_set_item_frame(source, FRAME_PITCHER);
	}

	//Play the avatar's animation
	script AVATAR
	{
		nohalt;
		call freeze;	//freeze them in place

		face directionFromAvatar(item);

		//Lean down to put the milk into the churn
		actor frame STAND;
		actor frame LEAN;	wait 3;
		actor frame STAND;	wait 1;

		//pump and grind
		repeat 3
		{
			wait 1; actor frame SWING_3;
			wait 1; actor frame USE;
			wait 1; actor frame SWING_2;
		};
	
		actor frame STAND;

		call unfreeze;
	}

	//Play the churn animation
	script item
	{
		wait 7;	//wait for the avatar to get up from putting the milk in

		frame FRAME_CHURN;
		repeat 3
		{
			sfx CHURN_SOUND;
			frame FRAME_CHURN_3; wait 1;
			frame FRAME_CHURN_2; wait 1;
			frame FRAME_CHURN;
		};

		//now, create the butter next to the churn
		call createButter;
	}
}

//Create the new butter object, beside the churn
createButter 0xB03()
{
	var butter;
	var target_pos;

	//create the new butter object
	butter = UI_create_new_object(SHAPE_FOOD);
	if (!butter) return;
	UI_set_item_frame(butter, FRAME_BUTTER);

	UI_set_item_flag(butter, TEMPORARY);
	UI_set_item_flag(butter, OKAY_TO_TAKE);

	target_pos = UI_get_object_position(item);
	//place it on the middle right-hand side of the churn
	target_pos[X] = target_pos[X] + 1;
	//target_pos[Y] = target_pos[Y] - 3;

	UI_update_last_created(target_pos);

	//Get a random party member to say something nice about butter
	if (UI_get_random(10) > 5) randomPartyBark("\"Mmmm, rich creamery butter.\"");
}


//go to the churn (called by Bottle and Bucket)
gotoChurn 0xB13(var churn, var milktype)
{
	//User tried to use a churn that was inside something
	if (UI_get_container(churn))
	{
		randomPartySay("\"Thou must first place the butter churn on the ground.\"");
		return;
	}

	UI_close_gumps();

	var offsetx = [-1, -1, -1, 0, 0];
	var offsety = [0, 1, -1, 1, -1];

	//go to the churn and call churnButter() once you get there
	gotoObject(churn, offsetx, offsety, -3, churnButter, churn, milktype);
}