//Mining machine - reimplemented to add slate behaviour for Rune of Honor quest

//quality flags for mining-machine production
enum machine_production
{
	MINES_BLACKROCK = 1,
	MINES_IRON		= 2,
	MINES_LEAD		= 3,
	MINES_SLATE		= 4
};

const int SOUND_MININGMACHINE			= 8;	//Boomboomboomboom
const int SOUND_CONVEYOR				= 16;	//Fzzzing sound
const int SOUND_MININGMACHINE_BROKEN	= 37;	//Tlllk
const int SOUND_MININGMACHINE_BROKEN2	= 76;	//Error beep

extern moveConveyor 0x60D();
MiningMachine 410()
{
	var conveyor;
	var quality;
	var ore_shapes;
	var chosen_shape;

	if (event != DOUBLECLICK) return;

	UI_close_gumps();

	conveyor	= UI_find_nearby(item, SHAPE_CONVEYOR_BELT, 10, 0);
	quality		= UI_get_item_quality(item);

	//machine is broken
	if (quality == 0 || quality > MINES_SLATE)
	{
		script item
		{
			frame 0;
			frame 1;
			sfx SOUND_MININGMACHINE_BROKEN;
			sfx SOUND_MININGMACHINE_BROKEN2;
		}
		return;
	}
	else
	{
		ore_shapes = [SHAPE_BLACKROCK, SHAPE_IRON_ORE, SHAPE_LEAD_ORE, SHAPE_STONE_CHIPS];
		chosen_shape = ore_shapes[quality];

		//Animate the mining machine blade spinning
		//Reformulated slightly differently from original animation (which used repeat2, and didn't loop frames intelligently)
		script item
		{
			repeat 28
			{
				next frame cycle;
				continue;
				sfx SOUND_MININGMACHINE;
			};
		}

		//Animate the conveyor belt moving along
		script conveyor
		{
			frame 0;

			repeat 24
			{
				next frame cycle;
				continue;
				sfx SOUND_CONVEYOR;
				call moveConveyor;	//This actually does the moving-along-Gertrude
			};
		}
	}
}

//called as part of MiningMachine animation script: creates ore, and moves objects upon the belt along one notch at a time
moveConveyor 0x60D()
{
	var mining_machine;
	var quality;
	var ore_shapes;
	var chosen_shape;
	var new_ore;

	var conveyor_pos;
	var conveyor_x;
	var conveyor_y;

	var objs_on_conveyor;
	var obj;
	var obj_pos;
	var obj_x;
	var obj_y;
	var obj_z;


	if (event != SCRIPTED) return;

	mining_machine	= UI_find_nearby(item, SHAPE_MINING_MACHINE, 10, 0);
	quality			= UI_get_item_quality(mining_machine);

	ore_shapes		= [SHAPE_BLACKROCK, SHAPE_IRON_ORE, SHAPE_LEAD_ORE, SHAPE_STONE_CHIPS];
	chosen_shape	= ore_shapes[quality];

	conveyor_pos	= UI_get_object_position(item);
	
	//creates an ore chunk
	if (UI_get_random(20) == 1)
	{
		new_ore = UI_create_new_object(chosen_shape);
		if (new_ore)
		{
			UI_set_item_flag(new_ore, TEMPORARY);

			if (chosen_shape == SHAPE_STONE_CHIPS) UI_set_item_frame(new_ore, FRAME_SLATE);
			else UI_set_item_frame(new_ore, 0);

			UI_update_last_created([conveyor_pos[X] - 9, conveyor_pos[Y], 1]);
		}
	}

	//shifts the ore chunks on the conveyor along

	//objects that are on the conveyor belt
	objs_on_conveyor = UI_find_nearby(item, SHAPE_ANY, 10, 0);
	conveyor_x = conveyor_pos[X];
	conveyor_y = conveyor_pos[Y];

	for (obj in objs_on_conveyor)
	{
		obj_pos = UI_get_object_position(obj);

		obj_x = obj_pos[X];
		obj_y = obj_pos[Y];
		obj_z = obj_pos[Z];

		//object is still on conveyorm, move it along one step
		if (obj_x <= conveyor_x && obj_y == conveyor_y && obj_z == 1)
		{
			if (UI_set_last_created(obj)) UI_update_last_created([obj_x + 1, obj_y, obj_z]);
		}

		//object is at the end of conveyor, drop it off
		else if (obj_x == conveyor_x + 1 && obj_y == conveyor_y && obj_z == 1)
		{
			if (UI_set_last_created(obj)) UI_update_last_created([obj_x, obj_y, 0]);
		}
	}
}