/* This contains general usecode functions for the Rune of Honor quest. Additional usecode for this quest may be found in:
constnats/bg_flags.uc	(new global flags)
npcs/finnigan.uc		(added conversations)
npcs/candice.uc			(added conversations)
npcs/julia.uc			(added conversations)
npcs/gladstone.uc		(added conversations)
npcs/lordbritish.uc		(added conversations)
items/miningmachine.uc	(slate production)

*/

const int RUNE_THEFT_TIMER = 16;			//ID of theft discovery timer: used with UI_get_timer() and UI_set_timer()
const int RUNE_THEFT_DISCOVERY_TIME = 8;	//How many game hours it takes for the theft to be discovered
const int RUNE_QUEST_XP = 100;				//The XP reward for properly completing the Rune quest

/* Triggered by usecode egg when the Rune of Honor (real or fake) is placed on the pedestal in the center of Trinsic */
TrinsicRunePlaced 0xD00()
{
	var rune;		//The rune that was placed on top of the egg
	var rune_frame;	//The frame of that rune

	//we'll have none of that double-click nonsense here, thankyou very much!
	if (event != EGG) return;

	//grab the Rune that was placed
	rune = UI_find_nearby(item, SHAPE_RUNE, 1, 0);	//must be within 1 unit of the egg
	if (rune) rune_frame = UI_get_item_frame(rune);	//check which rune it was

	//Real rune was placed
	if (rune_frame == FRAME_RUNE_HONOR)
	{
		gflags[REAL_RUNE_IN_TRINSIC] = true;

		//The player has placed the fake Rune in Britain, which means all conditions for successful completion are met: reward the player with XP
		if (gflags[FAKE_RUNE_IN_BRITAIN] &&	!gflags[RETURNED_STOLEN_RUNE] && !gflags[REWARDED_FOR_RUNE])
		{
			giveExperience(RUNE_QUEST_XP);
			gflags[REWARDED_FOR_RUNE] = true;
		}

		//The player has stolen the rune from Britain, without putting the replica Rune in its place
		//This starts the theft-discovery timer: after 8 hours, Finnigan will have received an angry letter from Britain demanding the Rune's return
		else if (!gflags[FAKE_RUNE_IN_BRITAIN] && !gflags[RUNE_THEFT_TIMER_RUNNING])
		{
			UI_set_timer(RUNE_THEFT_TIMER);
			gflags[RUNE_THEFT_TIMER_RUNNING] = true;	//stops timer restarting every time
		}
	}
	else
	{
		gflags[REAL_RUNE_IN_TRINSIC] = false;
	}

	//Fake rune was placed
	if (rune_frame == FRAME_RUNE_REPLICA) gflags[FAKE_RUNE_IN_TRINSIC] = true;
	else gflags[FAKE_RUNE_IN_TRINSIC] = false;
}

/* Triggered by usecode egg when the Rune of Honor (real or fake) is placed on the correct pedestal in the museum in Britain */
BritainRunePlaced 0xD01()
{

	var rune;
	var rune_frame;

	if (event != EGG) return;

	//grab the Rune that was placed
	rune = UI_find_nearby(item, SHAPE_RUNE, 1, 0);	//must be within 1 unit of the egg
	if (rune) rune_frame = UI_get_item_frame(rune);	//check which rune it was

	//Real rune was placed
	if (rune_frame == FRAME_RUNE_HONOR)
	{
		gflags[RUNE_TAKEN_FROM_BRITAIN] = false;

		//The player has returned the Rune after stealing it
		//This flags the quest as complete, because I can't be bothered writing "so did you get permission THIS time?" dialogs for people who retry the quest
		if (gflags[RUNE_THEFT_DISCOVERED])
		{
			gflags[RETURNED_STOLEN_RUNE]	= true;
			gflags[RUNE_QUEST_COMPLETE]		= true;
		}

		//Cancel theft discovery countdown
		else if (gflags[RUNE_THEFT_TIMER_RUNNING]) gflags[RUNE_THEFT_TIMER_RUNNING] = false;
	}
	else
	{
		gflags[RUNE_TAKEN_FROM_BRITAIN] = true;
	}

	//Fake rune was placed
	if (rune_frame == FRAME_RUNE_REPLICA)
	{
		gflags[FAKE_RUNE_IN_BRITAIN] = true;

		//The player has placed the real Rune in Trinsic, which means all conditions for successful completion are met: reward the player with XP
		if (gflags[REAL_RUNE_IN_TRINSIC] &&	!gflags[RETURNED_STOLEN_RUNE] && !gflags[REWARDED_FOR_RUNE])
		{
			giveExperience(RUNE_QUEST_XP);
			gflags[REWARDED_FOR_RUNE] = true;
		}

		//Player stole the original, but the theft hasn't been discovered yet
		//If the above is true, this certainly will be too
		if (gflags[RUNE_THEFT_TIMER_RUNNING] && !gflags[RUNE_THEFT_DISCOVERED])
		{
			gflags[RUNE_THEFT_TIMER_RUNNING] = false;
		}
	}
	else gflags[FAKE_RUNE_IN_BRITAIN] = false;
}

//Usecode egg in mayor's office in Trinsic: whenever player is near, checks if the theft has been discovered
//If so, makes him start conversation with player
checkTheftDiscovered 0xD02()
{
	if (event != EGG) return;

	var finnigan = UI_get_npc_object(FINNIGAN);

	//Theft has been discovered - have Finnigan go to the Avatar
	if (!gflags[RUNE_THEFT_DISCOVERED] && gflags[RUNE_THEFT_TIMER_RUNNING] && UI_get_timer(RUNE_THEFT_TIMER) > RUNE_THEFT_DISCOVERY_TIME)
	{
		UI_item_say(finnigan, "Avatar, I must speak to thee!\"");

		//run to the Avatar and start conversation when you get there
		UI_set_schedule_type(finnigan, TALK);
		return;
	}
}