/* This file contains include statements for all script modules. It should contain no actual usecode. */

/* The patch currently uses the following addresses for custom functions:

0xA00-0xAFF:	Generic functions (some of these use implicit addresses)
0xB00-0xBFF:	Item-specific functions
0xF00+:			Functions used in script blocks

This will change to use entirely implicit numbers once this is fully supported by the compiler.

If you use your own implicit addresses, bear in mind that they are assigned starting from the last explicit function definition. So, define an explicit dummy function at 0xC00 to start the numbering off there.
*/

//Header files
//------------
#include "headers/constants.uc";	//standard constant definitions
#include "headers/bg/bg_npcs.uc";	//Black Gate npc constants
#include "headers/bg/bg_shapes.uc";	//Black Gate shape and frame constants
#include "headers/bg/bg_gflags.uc";	//Black Gate global flags

#include "headers/bg/bg_externals.uc";	//extern declarations for BG functions
#include "headers/functions.uc";		//new general-purpose functions

#include "debug.uc";	//debugging and testing functions. Comment this out before release!

//Project Files
//-------------

#include "headers/new_gflags.uc";	//Newly added global flags
#include "headers/new_shapes.uc";	//Newly added shapes and frames

//Quests
#include "quests/honor.uc";			//Rune of Honor quest

//Items
#include "items/kitchenitem.uc";	//Miscellaneous kitchen items
#include "items/food.uc";			//Edible items
#include "items/baking.uc";			//Turning dough into bread
#include "items/bucket.uc";			//Buckets
#include "items/wheat.uc";			//Grinding wheat
//#include "items/key.uc";			//Keys
#include "items/cloth.uc";			//Cloth, thread and looms
#include "items/misc.uc";			//Miscellaneous interactions
#include "items/fishingrod.uc";		//Fishing rod behaviour
#include "items/pocketwatch.uc";	//Corrects am/pm bug
#include "items/shears.uc";			//Added sheep-shearing behaviour
#include "items/baby.uc";			//Allows you to put Lady Tory's baby into a cradle
#include "items/miningmachine.uc";	//Reimplemented for Rune of Honour quest

//Scripts used for forging
#include "items/bellows.uc";	//Bellows, reimplemented to shift off firepit behaviour to firepit.uc
#include "items/firepit.uc";	//Firepit, reimplemented to heat up swordblanks
#include "items/swordblank.uc";	//Swordblank, reimplemented to allow custom sword forging
#include "items/anvil.uc";		//Anvil, reimplemented to allow custom sword forging
#include "items/hammer.uc";		//Hammer, reimplemented to allow custom sword forging

//Monsters
#include "monsters/cow.uc";		//Cows!
#include "monsters/chicken.uc";	//Chickens!
#include "monsters/sheep.uc";	//Sheep!
#include "monsters/fish.uc";	//Fish!

//NPCs
#include "npcs/npcs.uc";			//generic NPC dialogue-related functions
#include "npcs/camille.uc";			//Camille the widow in Paws (adds a good deed)
#include "npcs/willy.uc";			//Willy the baker in Britain (fixes original conversation bug)
#include "npcs/gordon.uc";			//Gordon the fish-seller in Britain (adds fish-sales behaviour)
#include "npcs/finnigan.uc";		//Finnegan the mayor of Trinsic (adds Rune of Honour quest)
#include "npcs/candice.uc";			//Candice's the curator in Britain (adds Rune of Honour quest)
#include "npcs/lordbritish.uc";		//Lord B (adds Rune of Honour quest)
#include "npcs/julia.uc";			//Julia the tinker in Minoc (adds Rune of Honour quest)
#include "npcs/gladstone.uc";		//Gladstone the sculptor in Minoc (adds Rune of Honour quest)
#include "npcs/tory.uc";			//Lady Tory, the empath in Serpent's Hold (general dialogue improvements)
#include "npcs/menion.uc";			//Menion, the weaponsmith/trainer in Serpent's Hold (added swordmaking)
